
#include "sparseData.h"
#include <algorithm>
#include <fstream>
#include <sstream>
#include <stdexcept>

using namespace std;


//load sparse data files 
void loaddata(std::string filename, sparseData& data)
{
	size_t dataset_size;
    // first pass: find number of non-empty lines and entries
    size_t n = 0;
    size_t nnz = 0;
    {
        ifstream ifs(filename.c_str());
        if (! ifs.good()) throw runtime_error(string("[loaddata] failed to load file ").append(filename));
        while (true)
        {
            std::string s;
            if (! getline(ifs, s)) break;
            if (s.empty()) break;

            for (size_t i=0; i<s.size(); i++) if (s[i] == ':') nnz++;
            n++;
        }
    }

    // second pass: load contents
    {
        ifstream ifs(filename.c_str());
        if (! ifs.good()) throw runtime_error(string("[loaddata] failed to load file ").append(filename));
        for (std::size_t i=0; i<n; i++)
        {
            std::string s;
            if (! getline(ifs, s)) break;
            if (s.empty()) break;

            // extract label
            std::size_t start = 0;
            std::size_t end = std::min(s.find(' ', start), s.size());
            string y = s.substr(start, end - start);
            int i_dec = std::stoi (y);
            if (i_dec%2 == 0)
                data.labels.push_back((char)-1);
            else
                data.labels.push_back((char)strtol(y.c_str(), NULL, 10));
            for (start = end; start < s.size() && s[start] == ' '; start++);

            // extract features
			vector<SE> data_storage;
            while (start < s.size())
            {
                std::size_t colon = s.find(':', start);
                if (colon == std::string::npos) throw runtime_error(string("[loaddata] failed to load file ").append(filename));
                end = std::min(s.find(' ', colon), s.size());
                size_t feature = strtol(s.substr(start, colon - start).c_str(), NULL, 10) - 1;
                double value = strtod(s.substr(colon + 1, end - colon - 1).c_str(), NULL);
				data_storage.push_back(SE(feature, value));
                for (start = end; start < s.size() && s[start] == ' '; start++);
            }
			data_storage.push_back(SE(END(), 0.0));
			data.data.push_back(data_storage);
        }
    }
}

vector<SE> scaleAddSparseVectors(vector<SE> const& first_operand, vector<SE> const& second_operand, double first_scale, double second_scale) {
	vector<SE> result;

	size_t i = 0;
	size_t j = 0;
	size_t x1_size = first_operand.size();
	size_t x2_size = second_operand.size();
	INDEX index_i = (i >= x1_size) ? END():first_operand[i].index;
	INDEX index_j =  (j >= x2_size) ? END():second_operand[j].index;

	while (index_i != END() ||index_j != END()) {
		if (index_i < index_j) {
			result.push_back(SE(first_operand[i].index, first_operand[i].value*first_scale));
			i++;
		}

		if (index_i > index_j) {
			result.push_back(SE(second_operand[j].index, second_operand[j].value*second_scale));
			j++;
		}

		if (index_i == index_j) {
			result.push_back(SE(second_operand[j].index, first_operand[i].value*first_scale + second_operand[j].value*second_scale));
			i++;
			j++;
		}

		index_i = (i >= x1_size) ? END():first_operand[i].index;
		index_j =  (j >= x2_size) ? END():second_operand[j].index;
	}
	result.push_back(SE(END(), 0.0));
	return result;
}
